import 'dart:async';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/foundation.dart';
import 'dart:io';
import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/interfaces/mixins/online_status_mixin.dart';

class WowConnectivity with HandleExceptionMixin {
  WowConnectivity._() {
    Connectivity().onConnectivityChanged.listen(_checkConnection);
  }

  factory WowConnectivity() => _instance;

  static final WowConnectivity _instance = WowConnectivity._();

  /// | Address        | Provider   | Info                                            |
  /// |:---------------|:-----------|:------------------------------------------------|
  /// | 1.1.1.1        | CloudFlare | https://1.1.1.1                                 |
  /// | 1.0.0.1        | CloudFlare | https://1.1.1.1                                 |
  /// | 8.8.8.8        | Google     | https://developers.google.com/speed/public-dns/ |
  /// | 8.8.4.4        | Google     | https://developers.google.com/speed/public-dns/ |
  /// | 208.67.222.222 | OpenDNS    | https://use.opendns.com/                        |
  /// | 208.67.220.220 | OpenDNS    | https://use.opendns.com/                        |
  static final List<InternetAddress> addresses = List.unmodifiable([
    InternetAddress('1.1.1.1'),
    //InternetAddress('1.0.0.1'),
    InternetAddress('8.8.8.8'),
    //InternetAddress('8.8.4.4'),
    InternetAddress('208.67.222.222'),
    //InternetAddress('208.67.220.220'),
  ]);

  static const int port = 53;

  static const Duration timeout = Duration(seconds: 10);

  // static final Map<OnlineStatusMixin, StreamSubscription<ConnectivityResult>>
  //     _listSubscription = {};

  static final Set<OnlineStatusMixin> _list = {};

  bool _isOnline = true;
  bool get isOnline => _isOnline;

  // final Connectivity _connectivity = Connectivity();

  void initialise(OnlineStatusMixin controller) async {
    // var result = await _connectivity.checkConnectivity();
    // _checkStatus(result, controller);
    // _listSubscription[controller] =
    //     _connectivity.onConnectivityChanged.listen((connectivityResult) {
    //   _checkStatus(connectivityResult, controller);
    // });
    _list.add(controller);
    controller.isOnline = _instance._isOnline;
  }

  static void _checkConnection(ConnectivityResult connectivityResult) async {
    bool isOnline = true;
    try {
      bool checkHasConnect = connectivityResult != ConnectivityResult.none;
      bool checkHasInternet = await _instance.hasConnectionInternet();
      isOnline = checkHasConnect && checkHasInternet;
    } catch (e) {
      _instance.handelException(e, subTitle: '_checkConnection');
      isOnline = false;
    }
    _instance._isOnline = isOnline;
    for (var controller in _list) {
      if (kDebugMode) {
        print(
          'Connectivity reload connect ${controller.runtimeType}: isOnline = $isOnline',
        );
      }
      controller.isOnline = isOnline;
    }
  }

  void dispose(OnlineStatusMixin controller) {
    // if (_listSubscription.containsKey(baseController)) {
    //   _listSubscription[baseController]!.cancel();
    // }
    _list.remove(controller);
  }

  // void _checkStatus(ConnectivityResult connectivityResult,
  //     OnlineStatusMixin controller) async {
  //   bool isOnline = true;
  //   try {
  //     final result = await InternetAddress.lookup('google.com');
  //     if (connectivityResult != ConnectivityResult.none &&
  //         result.isNotEmpty &&
  //         result[0].rawAddress.isNotEmpty) {
  //       try {
  //         isOnline = true;
  //       } catch (e) {
  //         if (kDebugMode) {
  //           print('Exception _checkStatus: ' + e.toString());
  //         }
  //         await FirebaseService.logout();
  //         await Get.offAll(() => Common.registerScreen);
  //       }
  //     } else {
  //       isOnline = false;
  //     }
  //   } on SocketException catch (e) {
  //     if (kDebugMode) {
  //       print('Exception _checkStatus: ' + e.toString());
  //     }
  //     isOnline = false;
  //   }
  //   if (_isOnline != isOnline) {
  //     _isOnline = isOnline;
  //   }
  //   if (kDebugMode) {
  //     print(
  //         'Connectivity reload connect ${controller.runtimeType}: isOnline = $_isOnline');
  //   }
  //   controller.isOnline = isOnline;
  // }

  Future<bool> hasConnectionInternet() async {
    List<Future<bool>> requests = [];
    for (var address in addresses) {
      requests.add(isHostReachable(address));
    }
    var listCheck = await Future.wait(requests);

    return listCheck.contains(true);
  }

  Future<bool> isHostReachable(
    InternetAddress host,
  ) async {
    Socket? sock;
    try {
      sock = await Socket.connect(
        host,
        port,
        timeout: timeout,
      );
      sock.destroy();
      if (kDebugMode) {
        print('connect Internet Address: $host = true');
      }
      return true;
    } catch (e) {
      sock?.destroy();
      if (kDebugMode) {
        print('connect Internet Address: $host = false');
      }
      return false;
    }
  }
}
