import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/common.dart';
import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/sql_service.dart';

mixin MessageInitMixin on HandleExceptionMixin {
  static const String _prefMessageKey = 'prefMessage';

  static const String _splitValue = '[..]';

  String procMessage = 'getMessage';
  String tableMessage = 'language';

  Future initMessage({
    bool reload = false,
    int? languageId,
  }) async {
    try {
      languageId ??= Common.languageId;
      Map<String, String> mapMessage = await loadMessageFromLocal();
      if (mapMessage.isEmpty || reload) {
        mapMessage = await loadMessageFromSQL(languageId: languageId);
        unawaited(saveMessageToLocal(map: mapMessage));
        unawaited(updateHasGetMessage());
      }
      Common.mapMessageData = mapMessage;
    } catch (e) {
      handelException(e, subTitle: 'initMessage');
    }
  }

  Future saveMessageToLocal({
    required Map<String, String> map,
  }) async {
    try {
      SharedPreferences pref = await SharedPreferences.getInstance();
      List<String> list = map.entries
          .map<String>((e) => '${e.key}$_splitValue${e.value}')
          .toList();
      await pref.setStringList(_prefMessageKey, list);
      if (kDebugMode) {
        print(
          '$runtimeType.saveMessageToLocal success: ${list.length} message',
        );
      }
    } catch (e) {
      handelException(e, subTitle: 'saveMessageToLocal');
    }
  }

  Future<Map<String, String>> loadMessageFromLocal() async {
    try {
      SharedPreferences pref = await SharedPreferences.getInstance();
      List<String> list = pref.getStringList(_prefMessageKey) ?? [];
      Map<String, String> map = {
        for (String item in list)
          item.split(_splitValue).first: item.split(_splitValue).last,
      };
      if (kDebugMode) {
        print(
          '$runtimeType.loadMessageFromLocal success: ${map.length} message',
        );
      }
      return map;
    } catch (e) {
      handelException(e, subTitle: 'loadMessageFromLocal');
      return {};
    }
  }

  Future<Map<String, String>> loadMessageFromSQL({
    required int languageId,
  }) async {
    Map<String, String> map = {};
    try {
      Map<String, Object> params = {
        'uid': Common.uid,
        'languageId': languageId,
      };
      var data = await SQLService.execute(
        procMessage,
        params: params,
      );
      var list = Methods.getList(data, tableMessage);
      for (var item in list) {
        String key = Methods.getString(item, 'messageId');
        String message = Methods.getString(item, 'message');
        map.putIfAbsent(key, () => message);
      }
      return map;
    } catch (e) {
      handelException(e, subTitle: 'loadMessageFromSQL');
      return map;
    }
  }

  Future updateHasGetMessage() async {
    Map<String, Object> params = {
      'uid': Common.uid,
      'hasGetMessage': true,
    };
    await SQLService.execute(
      'sp_setHasMapData',
      params: params,
    );
  }
}
